#!/usr/bin/env php
<?php

class Curly
{
    private $Curly;
    private $cURL;
    private $argv;
    private $xmlconfig;
    private $CurlOptStdErrHandle;
    private $CurlOptFileHandle;
    private $LogHandle;

    public function CurlyExec($argv)
    {
        $this->argv = $argv;
        $this->LoadDefaultCnf();
        $this->CurlyPreChecks();
        $this->LoadXmlCnf();
        $this->cookies();
        $this->RunTestCases();
        $this->endtasks();
        if ($this->Curly['conf']['debug'] == 1) {
            print_r($this->Curly);
        }
        echo $this->Curly['result']['output']['stdout'] . "\n";
        exit($this->Curly['result']['exit_status']);
    }

    private function LoadDefaultCnf()
    {
        $this->Curly['conf'] = array(
            'testcases' => array(),
            'exit_codes' => array(
                'UNKNOWN' => 3,
                'OK' => 0,
                'WARNING' => 1,
                'CRITICAL' => 2,
            ),
            'curl_setopt' => array(
                'CURLOPT_RETURNTRANSFER' => 1, /* TRUE to return the transfer as a string of the return value of curl_exec() instead of outputting it out directly. */
                'CURLOPT_USERAGENT' => 'Curly',
                'CURLOPT_VERBOSE' => 1, /* TRUE to output verbose information. Writes output to STDERR, or the file specified using CURLOPT_STDERR. */
                'CURLOPT_FOLLOWLOCATION' => 1, /* TRUE to follow any "Location: " header that the server sends as part of the HTTP header (note this is recursive, PHP will follow as many "Location: " headers that it is sent, unless CURLOPT_MAXREDIRS is set). */
                'CURLOPT_CONNECTTIMEOUT' => 10, /* The number of seconds to wait while trying to connect. Use 0 to wait indefinitely.  */
                'CURLOPT_TIMEOUT' => 10, /* The maximum number of seconds to allow cURL functions to execute. */
                'CURLOPT_STDERR' => 'php://memory',
                'CURLOPT_COOKIEFILE' => '',
                'CURLOPT_COOKIEJAR' => '',
            ),
            'curl_setopt_deny' => array('CURLOPT_RETURNTRANSFER', /* CURLOPT_RETURNTRANSFER value can not be overriden for Curly to work */
                'CURLOPT_VERBOSE', /* CURLOPT_RETURNTRANSFER value can not be overriden for Curly to work */
                /* 'CURLOPT_FILE', The file that the transfer should be written to. The default is STDOUT (the browser window). */
                'CURLOPT_HEADERFUNCTION',
                'CURLOPT_WRITEFUNCTION',
                'CURLOPT_WRITEHEADER', /* The file that the header part of the transfer is written to. */
                'CURLOPT_READFUNCTION',
                'CURLOPT_COOKIEFILE',
                'CURLOPT_COOKIEJAR',
                'CURLOPT_STDERR'),
            'case_name_filter' => '/^[a-zA-Z0-9_-]+$/',
            'errormessage_filter' => '/^[a-zA-Z0-9_-]+$/',
            'flags_ok' => array(0, 1),
            'debug' => 0,
            'break_on_error' => 1,
            'logenable' => 1,
            'logfile' => 'Curly.log',
            'deletecookies' => 1,
            'cookiesdir' => '/tmp',
            'output_ok_length' => 'long',
            'logsdir' => '/var/log/Curly',
            'checks' => array('status_code',
                'matchpattern',
                'nomatchpattern',
                'matchpatterncount',
                'parsepattern',
                'primary_ip',
                'redirect_count',
                'response_time',
                'md5sum'),
        );
        $this->Curly['version'] = '1.0';
        $this->Curly['curlopt_constants'] = $this->curlopt_constants();
        $this->Curly['curlauth_constants'] = $this->curlauth_constants();
        $this->Curly['curlinfo_constants'] = $this->curlinfo_constants();
        $this->Curly['parsepattern_results'] = array();
        $this->Curly['result'] = array(
            'output' => array(
                'UNKNOWN' => '',
                'OK' => '',
                'WARNING' => '',
                'CRITICAL' => '',
                'perfdatas' => '',
                'stdout' => '',
            ),
            'counters' => array(
                'CurlyStartTime' => microtime(true),
                'CurlyEndTime' => 0,
                'CurlyTime' => 0,
                'casesrun' => 0,
                'casestime' => 0,
                'casesok' => 0,
                'casescritical' => 0,
                'caseswarning' => 0,
                'casesunknown' => 0,
                'checksrun' => 0,
                'checksok' => 0,
                'checkscritical' => 0,
                'checksunknown' => 0,
                'checkswarning' => 0,
                'checkscritical' => 0,
            ),
            'exit_status' => 0,
            'testcases' => array(),
        );
    }

    private function curlopt_constants()
    {
        $curlopt_constants = array();
        foreach (get_defined_constants() as $name => $val) {
            if (strpos($name, 'CURLOPT_') === 0) {
                $curlopt_constants[$name] = $val;
            }
        }
        return $curlopt_constants;
    }

    private function curlauth_constants()
    {
        $curlauth_constants = array();
        foreach (get_defined_constants() as $name => $val) {
            if (strpos($name, 'CURLAUTH_') === 0) {
                $curlauth_constants[$name] = $val;
            }
        }
        return $curlauth_constants;
    }

    private function curlinfo_constants()
    {
        $curlinfo_constants = array();
        foreach (get_defined_constants() as $name => $val) {
            if (strpos($name, 'CURLINFO_') === 0) {
                $curlinfo_constants[$name] = $val;
            }
        }
        return $curlinfo_constants;
    }

    private function CurlyPreChecks()
    {
        if (version_compare(PHP_VERSION, '5.1.0', '<')) {
            echo 'Curly UNKNOWN - PHP version >= 5.1.0 required - PHP version is ' . PHP_VERSION . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        if (!extension_loaded('curl')) {
            echo 'Curly UNKNOWN - PHP cURL extension not loaded' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        if (!extension_loaded('json')) {
            echo 'Curly UNKNOWN - PHP json extension not loaded' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        if (count($this->argv) < 2 || count($this->argv) > 2) {
            echo 'Curly UNKNOWN - Usage: ' . $this->argv[0] . ' config.xml' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        if ($this->argv[1] == '') {
            echo 'Curly UNKNOWN - Usage: ' . $this->argv[0] . ' config.xml' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        if ($this->argv[1] == '-h') {
            echo 'Usage: ' . $this->argv[0] . ' config.xml' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['OK']);
        }
        if ($this->argv[1] == '-v') {
            echo $this->argv[0] . ' version ' . $this->Curly['version'] . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['OK']);
        }
        if (!file_exists($this->argv[1])) {
            echo 'Curly UNKNOWN - File ' . $this->argv[1] . ' not found' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        $this->xmlconfig = $this->argv[1];
    }

    private function LoadXmlCnf()
    {
        $output_ok_length_ok = array('short', 'long');
        $XmlCnf = simplexml_load_file($this->xmlconfig, null, LIBXML_NOCDATA);
        if (!$XmlCnf) {
            echo 'Curly UNKNOWN - Failed loading ' . $this->xmlconfig . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }

        if (empty($XmlCnf->testcases->testcase)) {
            echo 'Curly UNKNOWN - No testcases found ' . $this->xmlconfig . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }

        $XmlCnf = $this->XmlToArray($XmlCnf);

        if (is_array($XmlCnf['testcases']['testcase'])) {
            foreach ($XmlCnf['testcases']['testcase'] as $key => $testcasefile) {
                if (in_array($testcasefile, $this->Curly['conf']['testcases'])) {
                    echo 'Curly UNKNOWN - ' . $testcasefile . ' already used' . PHP_EOL;
                    exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                }
                array_push($this->Curly['conf']['testcases'], $testcasefile);
            }
        } else {
            $testcasefile = $XmlCnf['testcases']['testcase'];
            array_push($this->Curly['conf']['testcases'], $testcasefile);
        }

        if (isset($XmlCnf['break_on_error'])) {
            if (in_array($XmlCnf['break_on_error'], $this->Curly['conf']['flags_ok'])) {
                $this->Curly['conf']['break_on_error'] = $XmlCnf['break_on_error'];
            }
        }
        if (isset($XmlCnf['logenable'])) {
            if (in_array($XmlCnf['logenable'], $this->Curly['conf']['flags_ok'])) {
                $this->Curly['conf']['logenable'] = $XmlCnf['logenable'];
            }
        }

        if (isset($XmlCnf['logsdir'])) {
            if (empty($XmlCnf['logsdir']) or is_array($XmlCnf['logsdir'])) {
                echo 'Curly UNKNOWN - logsdir error in ' . $this->xmlconfig . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }
            if (!is_dir($XmlCnf['logsdir'])) {
                echo 'Curly UNKNOWN - Directory ' . $XmlCnf['logsdir'] . ' not found' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);

            }
            if (!is_writable($XmlCnf['logsdir'])) {
                echo 'Curly UNKNOWN - Directory ' . $XmlCnf['logsdir'] . ' not writable' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);

            }
            $XmlCnf['logsdir'] = preg_replace('/\/$/', '', $XmlCnf['logsdir']);
            $this->Curly['conf']['logsdir'] = $XmlCnf['logsdir'];
        }

        if (!is_dir($this->Curly['conf']['logsdir'])) {
            echo 'Curly UNKNOWN - Directory ' . $this->Curly['conf']['logsdir'] . ' not found' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        if (!is_writable($this->Curly['conf']['logsdir'])) {
            echo 'Curly UNKNOWN - Directory ' . $this->Curly['conf']['logsdir'] . ' not writable' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }

        if (isset($XmlCnf['output_ok_length'])) {
            if (in_array($XmlCnf['output_ok_length'], $output_ok_length_ok)) {
                $this->Curly['conf']['output_ok_length'] = $XmlCnf['output_ok_length'];
            }
        }

        if (isset($XmlCnf['logfile'])) {
            if (!empty($XmlCnf['logfile']) && !is_array($XmlCnf['logfile']) && !preg_match('/\//', $XmlCnf['logfile'], $matches)) {
                $this->Curly['conf']['logfile'] = $XmlCnf['logfile'];
            } else {
                echo 'Curly UNKNOWN - logfile error in ' . $this->xmlconfig . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }
        } else {
            $logfile = basename($this->xmlconfig);
            $logfile = preg_replace('/.xml/', '.log', $logfile);
            $this->Curly['conf']['logfile'] = $logfile;
        }

        if (isset($XmlCnf['deletecookies'])) {
            if (in_array($XmlCnf['deletecookies'], $this->Curly['conf']['flags_ok'])) {
                $this->Curly['conf']['deletecookies'] = $XmlCnf['deletecookies'];
            }
        }

        if (isset($XmlCnf['debug'])) {
            if (in_array($XmlCnf['debug'], $this->Curly['conf']['flags_ok'])) {
                $this->Curly['conf']['debug'] = $XmlCnf['debug'];
            }
        }

        if (isset($XmlCnf['curl_setopt'])) {
            if (is_array($XmlCnf['curl_setopt'])) {
                foreach ($XmlCnf['curl_setopt'] as $XmlCnfCurlopt => $XmlCnfCurloptvalue) {
                    if (!is_array($XmlCnfCurlopt)) {
                        if (!in_array($XmlCnfCurlopt, $this->Curly['conf']['curl_setopt_deny'])) {
                            if (isset($this->Curly['curlopt_constants'][$XmlCnfCurlopt])) {
                                if (!is_array($XmlCnfCurloptvalue)) {
                                    $this->Curly['conf']['curl_setopt'][$XmlCnfCurlopt] = $XmlCnfCurloptvalue;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private function XmlToArray($Xml)
    {
        $json = json_encode($Xml);
        $array = json_decode($json, TRUE);
        return $array;
    }

    private function cookies()
    {
        if (!is_dir($this->Curly['conf']['cookiesdir'])) {
            echo 'Curly UNKNOWN - Directory ' . $this->Curly['conf']['cookiesdir'] . ' - not found' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        if (!is_writable($this->Curly['conf']['cookiesdir'])) {
            echo 'Curly UNKNOWN - Directory ' . $this->Curly['conf']['cookiesdir'] . ' - not writable' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }

        $character_set_array = array();
        $character_set_array[] = array('count' => 7, 'characters' => 'abcdefghijklmnopqrstuvwxyz');
        $character_set_array[] = array('count' => 1, 'characters' => '0123456789');
        $temp_array = array();
        foreach ($character_set_array as $character_set) {
            for ($i = 0; $i < $character_set['count']; $i++) {
                $temp_array[] = $character_set['characters'][rand(0, strlen($character_set['characters']) - 1)];
            }
        }
        shuffle($temp_array);
        $cookie = implode('', $temp_array);
        $this->Curly['conf']['curl_setopt']['CURLOPT_COOKIEFILE'] = $this->Curly['conf']['cookiesdir'] . '/Curly_' . $cookie . '.cookie';
        $this->Curly['conf']['curl_setopt']['CURLOPT_COOKIEJAR'] = $this->Curly['conf']['cookiesdir'] . '/Curly_' . $cookie . '.jar';
    }

    private function RunTestCases()
    {
        foreach ($this->Curly['conf']['testcases'] as $casefilekey => $testcasefile) {

            $cases = $this->LoadXmlTestCase($testcasefile);
            $testcasefile = basename($testcasefile);
            $cases = $this->LoadCases($testcasefile, $cases);

            foreach ($cases as $casekey => $case) {
                $casename = $case['@attributes']['name'] . ' ';
                $this->RunCase($testcasefile, $casekey, $case);

                if ($this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] == 1) {
                    $this->Curly['result']['counters']['casesunknown']++;
                    $this->Curly['result']['output']['UNKNOWN'] .= $casename . $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'];
                    return;
                } elseif ($this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] == 1) {
                    $this->Curly['result']['counters']['casescritical']++;
                    $this->Curly['result']['output']['CRITICAL'] .= $casename . $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'];
                    if ($this->Curly['conf']['break_on_error'] == 1) {
                        return;
                    }
                } elseif ($this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] == 1) {
                    $this->Curly['result']['counters']['caseswarning']++;
                    $this->Curly['result']['output']['WARNING'] .= $casename . $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'];
                    if ($this->Curly['conf']['break_on_error'] == 1) {
                        return;
                    }
                } else {
                    $this->Curly['result']['counters']['casesok']++;
                    $this->Curly['result']['output']['OK'] .= $casename . $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'];
                }
            }
        }
        return;
    }

    private function LoadXmlTestCase($testcasefile)
    {
        $cases = simpleXml_load_file($testcasefile, null, LIBXML_NOCDATA);
        if (!$cases) {
            echo 'Curly UNKNOWN - Failed loading ' . $testcasefile . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }
        return $cases;
    }

    private function LoadCases($testcasefile, $cases)
    {
        $this->CheckCases($testcasefile, $cases);
        $cases = $this->XmlToArray($cases);
        $sortcases = array();

        if (isset($cases['case']['@attributes'])) {
            array_push($sortcases, $cases['case']);
        } else {
            # Sort cases by attributes ids
            $attributeids = array();
            $sortattributeids = array();
            foreach ($cases['case'] as $key => $val) {
                $attributeid = $cases['case'][$key]['@attributes']['id'];
                array_push($attributeids, $attributeid);
            }
            asort($attributeids);
            foreach ($attributeids as $key => $val) {
                array_push($sortattributeids, $val);
            }
            foreach ($sortattributeids as $k1 => $v1) {
                foreach ($cases['case'] as $key => $val) {
                    if ($cases['case'][$key]['@attributes']['id'] == $v1) {
                        array_push($sortcases, $cases['case'][$key]);
                    }
                }
            }
        }
        # Dirty hack to get same array depth for all checks
        foreach ($sortcases as $key => $val) {
            foreach ($val['checks'] as $check => $arg) {
                foreach ($arg as $ka => $kv) {
                    if (!is_array($kv)) {
                        $sortcases[$key]['checks'][$check][0] = $arg;
                        unset($sortcases[$key]['checks'][$check][$ka]);
                    }
                }
            }
        }

        return $sortcases;
    }

    private function CheckCases($testcasefile, $cases)
    {
        $check_caseids = array();
        $check_casenames = array();
        $errormessage_prefix = 'Curly UNKNOWN - ' . $testcasefile . ' - ';
        $RootNode = $cases->getName();
        $sources_ok = array('curl', 'curlverbose');
        $onfail_status_ok = array('WARNING', 'CRITICAL');

        if ($RootNode != 'cases') {
            echo $errormessage_prefix;
            echo 'root element <' . $RootNode . '> not recognized - use <cases> instead' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }

        if (!isset($cases->case)) {
            echo $errormessage_prefix;
            echo 'no case found' . PHP_EOL;
            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
        }

        foreach ($cases->case as $case) {
            if (empty($case->attributes()->id)) {
                echo $errormessage_prefix;
                echo 'a case id attribute is missing' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            if (!preg_match('/^[0-9]+$/', $case->attributes()->id, $matches)) {
                echo $errormessage_prefix;
                echo 'case ' . $case->attributes()->id . ' - not an integer' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            $caseid = (int)$case->attributes()->id;

            if (in_array($caseid, $check_caseids)) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' is already used' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }
            array_push($check_caseids, $caseid);

            if (empty($case->attributes()->name)) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' - name attribute missing' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            $casename = (string)$case->attributes()->name;
            if (!preg_match($this->Curly['conf']['case_name_filter'], $casename, $matches)) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' - case name format filter - ' . $this->Curly['conf']['case_name_filter'] . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }
            if (in_array($casename, $check_casenames)) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' - case ' . $casename . ' already used' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }
            array_push($check_casenames, $casename);

            if (count($case->children()) == 0) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' - no xml elements found' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            if (empty($case->curl_setopt)) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' - curl_setopt element not found' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            if (empty($case->curl_setopt->CURLOPT_URL)) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' - CURLOPT_URL element not found' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            if (empty($case->checks) or count($case->checks->children()) == 0) {
                echo $errormessage_prefix;
                echo 'case' . $caseid . ' - no checks defined' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            # Unique checks
            $check_status_code = array();
            $check_primary_ip = array();
            $check_redirect_count = array();
            $check_response_time = array();

            foreach ($case->checks->children() as $checkname => $checkarg) {

                $checkname = trim((string)$checkname);

                if (!in_array($checkname, $this->Curly['conf']['checks'])) {
                    echo $errormessage_prefix;
                    echo 'case' . $caseid . ' - unknown check ' . $checkname . PHP_EOL;
                    exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                }
                if ($checkname == 'status_code') {
                    if (in_array($checkname, $check_status_code)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - already defined' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (empty($checkarg->code)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - code element required' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (count($checkarg->code) != 1) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - one code element only' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!preg_match('/^[0-9]+$/', $checkarg->code, $matches)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - code ' . $checkarg->code . ' not an integer' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (isset($checkarg->onfail_status)) {
                        if (empty($checkarg->onfail_status)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                        $status = trim((string)$checkarg->onfail_status);
                        if (!in_array($status, $onfail_status_ok)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    if (isset($checkarg->errormessage)) {
                        $errormessage = trim((string)$checkarg->errormessage);
                        if (!preg_match($this->Curly['conf']['errormessage_filter'], $errormessage, $matches)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' errormessage format filter - ' . $this->Curly['conf']['errormessage_filter'] . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    array_push($check_status_code, $checkname);
                } elseif ($checkname == 'primary_ip') {
                    if (in_array($checkname, $check_primary_ip)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - already defined' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (empty($checkarg->ip)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - ip element required' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (count($checkarg->ip) != 1) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - one ip element only' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (isset($checkarg->onfail_status)) {
                        if (empty($checkarg->onfail_status)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                        $status = trim((string)$checkarg->onfail_status);
                        if (!in_array($status, $onfail_status_ok)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    if (isset($checkarg->errormessage)) {
                        $errormessage = trim((string)$checkarg->errormessage);
                        if (!preg_match($this->Curly['conf']['errormessage_filter'], $errormessage, $matches)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' errormessage format filter - ' . $this->Curly['conf']['errormessage_filter'] . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    array_push($check_primary_ip, $checkname);
                } elseif ($checkname == 'redirect_count') {
                    if (in_array($checkname, $check_redirect_count)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - already defined' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!isset($checkarg->count)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - count element required' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (count($checkarg->count) != 1) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - one count element only' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!preg_match('/^[0-9]+$/', $checkarg->count, $matches)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - count ' . $checkarg->count . ' not an integer' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (isset($checkarg->onfail_status)) {
                        if (empty($checkarg->onfail_status)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                        $status = trim((string)$checkarg->onfail_status);
                        if (!in_array($status, $onfail_status_ok)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    if (isset($checkarg->errormessage)) {
                        $errormessage = trim((string)$checkarg->errormessage);
                        if (!preg_match($this->Curly['conf']['errormessage_filter'], $errormessage, $matches)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' errormessage format filter - ' . $this->Curly['conf']['errormessage_filter'] . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    array_push($check_redirect_count, $checkname);
                } elseif ($checkname == 'response_time') {
                    if (in_array($checkname, $check_response_time)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - already defined' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (empty($checkarg->time)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - time element required' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (count($checkarg->time) != 1) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - one time element only' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!preg_match('/^[0-9]+$/', $checkarg->time, $matches)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - time ' . $checkarg->time . ' not an integer' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (isset($checkarg->onfail_status)) {
                        if (empty($checkarg->onfail_status)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                        $status = trim((string)$checkarg->onfail_status);
                        if (!in_array($status, $onfail_status_ok)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    if (isset($checkarg->errormessage)) {
                        $errormessage = trim((string)$checkarg->errormessage);
                        if (!preg_match($this->Curly['conf']['errormessage_filter'], $errormessage, $matches)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' errormessage format filter - ' . $this->Curly['conf']['errormessage_filter'] . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    array_push($check_response_time, $checkname);
                } elseif ($checkname == 'matchpatterncount') {
                    if (empty($checkarg->pattern) or empty($checkarg->source) or empty($checkarg->matchcount)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - pattern source matchcount elements required' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!preg_match('/^\//', $checkarg->pattern, $matches)) {
                        print_r($matches);
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - pattern ' . $checkarg->pattern . ' - Delimiters and optional pattern modifiers must be set in pattern element' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (count($checkarg->pattern) != 1 or count($checkarg->source) != 1 or count($checkarg->matchcount) != 1) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - one pattern source matchcount elements only' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!preg_match('/^[0-9]+$/', $checkarg->matchcount, $matches)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - matchcount ' . $checkarg->matchcount . ' not an integer' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!in_array($checkarg->source, $sources_ok)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - source curl/curlverbose' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (isset($checkarg->onfail_status)) {
                        if (empty($checkarg->onfail_status)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                        $status = trim((string)$checkarg->onfail_status);
                        if (!in_array($status, $onfail_status_ok)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    if (isset($checkarg->errormessage)) {
                        $errormessage = trim((string)$checkarg->errormessage);
                        if (!preg_match($this->Curly['conf']['errormessage_filter'], $errormessage, $matches)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' errormessage format filter - ' . $this->Curly['conf']['errormessage_filter'] . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                } else {
                    if (empty($checkarg->pattern) or empty($checkarg->source)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - pattern and source elements required' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (count($checkarg->pattern) != 1 or count($checkarg->source) != 1) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - one pattern source elements only' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!preg_match('/^\//', $checkarg->pattern, $matches)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - pattern ' . $checkarg->pattern . ' - Delimiters and optional pattern modifiers must be set in pattern element' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (!in_array($checkarg->source, $sources_ok)) {
                        echo $errormessage_prefix;
                        echo 'case' . $caseid . ' - ' . $checkname . ' - source curl/curlverbose' . PHP_EOL;
                        exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                    }
                    if (isset($checkarg->onfail_status)) {
                        if (empty($checkarg->onfail_status)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                        $status = trim((string)$checkarg->onfail_status);
                        if (!in_array($status, $onfail_status_ok)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' - onfail_status WARNING/CRITICAL' . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }
                    }
                    if (isset($checkarg->errormessage)) {
                        $errormessage = trim((string)$checkarg->errormessage);
                        if (!preg_match($this->Curly['conf']['errormessage_filter'], $errormessage, $matches)) {
                            echo $errormessage_prefix;
                            echo 'case' . $caseid . ' - ' . $checkname . ' errormessage format filter - ' . $this->Curly['conf']['errormessage_filter'] . PHP_EOL;
                            exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
                        }

                    }
                }
            }
        }
    }

    private function RunCase($testcasefile, $casekey, $case)
    {

        $caseid = $case['@attributes']['id'];
        $casename = $case['@attributes']['name'];
        $this->Curly['result']['counters']['casesrun']++;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['caseid'] = $caseid;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['casename'] = $casename;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 0;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 0;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 0;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] = '';
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] = '';
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] = '';
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] = '';

        $this->cURL = curl_init();

        $this->CurlyCurlSetOpt($testcasefile, $casekey, $case, $caseid);

        if ($this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] == 1) {
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo'] = array();
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'] = array();
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl'] = 'CurlyCurlSetOpt failed';
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['curlverbose'] = 'CurlyCurlSetOpt failed';

            if (gettype($this->CurlOptStdErrHandle) == 'resource') {
                fclose($this->CurlOptStdErrHandle);
            }
            if (gettype($this->CurlOptFileHandle) == 'resource') {
                fclose($this->CurlOptFileHandle);
            }
            curl_close($this->cURL);
            return;
        }

        $cURLExec = curl_exec($this->cURL);
        $cURLExec_curl_errno = curl_errno($this->cURL);
        $cURLExec_curl_error = curl_error($this->cURL);
        $cURLExec_curl_getinfo = curl_getinfo($this->cURL);

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_errno'] = $cURLExec_curl_errno;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl'] = $cURLExec;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo'] = $cURLExec_curl_getinfo;
        $casetime = substr($this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['total_time'], 0, 5);
        $this->Curly['result']['counters']['casestime'] += $casetime;

        /*'label'=value[UOM];[warn];[crit];[min];[max]*/
        /* Without response_time check we set casename perfdatas with null warn;crit;min;max */
        if (!isset($case['checks']['response_time'])) {
            $this->Curly['result']['output']['perfdatas'] .= $casename . '=' . $casetime . ';0;0;0;0 ';
        }

        if (gettype($this->CurlOptFileHandle) == 'resource') {
            fclose($this->CurlOptFileHandle);
        }

        rewind($this->CurlOptStdErrHandle);
        $CurlVerboseInf = stream_get_contents($this->CurlOptStdErrHandle);
        fclose($this->CurlOptStdErrHandle);
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['curlverbose'] = $CurlVerboseInf;

        if ($cURLExec === false) {
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'] = array();
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl'] = $cURLExec_curl_error;
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] = $cURLExec_curl_error;
            curl_close($this->cURL);
            return;
        }

        $this->ExecChecks($testcasefile, $casekey, $case, $caseid);
        curl_close($this->cURL);
        return;
    }

    private function CurlyCurlSetOpt($testcasefile, $casekey, $case, $caseid)
    {

        $this->CurlOptStdErrHandle = false;
        $this->CurlOptFileHandle = false;

        foreach ($this->Curly['conf']['curl_setopt'] as $Curlopt => $Curloptvalue) {
            $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_setopt'][$Curlopt] = $Curloptvalue;
            if ($this->Curly['conf']['debug'] == 1) {
                echo $Curlopt . PHP_EOL;
                print_r($Curloptvalue) . PHP_EOL;
                echo PHP_EOL;
            }
            if ($Curlopt == 'CURLOPT_STDERR') {
                /* Open for reading and writing; place the file pointer at the beginning of the file and truncate the file to zero length. If the file does not exist, attempt to create it. */
                if (!$this->CurlOptStdErrHandle = fopen($Curloptvalue, 'w+')) {
                    $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                    $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] = $Curlopt . ' - ' . $Curloptvalue . ' fopen failed';
                    return;
                } else {
                    curl_setopt($this->cURL, $this->Curly['curlopt_constants'][$Curlopt], $this->CurlOptStdErrHandle);
                }
            } elseif ($Curlopt == 'CURLOPT_HTTPHEADER') {
                $httpheaders = explode('|', $Curloptvalue);
                if ($this->Curly['conf']['debug'] == 1) {
                    echo $Curlopt . PHP_EOL;
                    print_r($Curloptvalue) . PHP_EOL;
                    print_r($httpheaders);
                    echo PHP_EOL;
                }
                curl_setopt($this->cURL, $this->Curly['curlopt_constants'][$Curlopt], $httpheaders);
            } else {
                curl_setopt($this->cURL, $this->Curly['curlopt_constants'][$Curlopt], $Curloptvalue);
            }
        }

        foreach ($case['curl_setopt'] as $Curlopt => $Curloptvalue) {

            if (isset($this->Curly['curlopt_constants'][$Curlopt])) {

                if (!in_array($Curlopt, $this->Curly['conf']['curl_setopt_deny'])) {

                    if ($this->Curly['conf']['debug'] == 1) {
                        echo $Curlopt . PHP_EOL;
                        print_r($Curloptvalue) . PHP_EOL;
                        echo PHP_EOL;
                    }

                    if (preg_match_all('/{get_parsepattern:/', $Curloptvalue, $matches)) {
                        $requests = count($matches[0]);
                        list($Curloptvalue, $status, $message) = $this->get_parsepattern($testcasefile, $caseid, $Curlopt, $Curloptvalue, $requests);
                        if ($status == 3) {
                            $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                            $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] = $message;
                            return;
                        }
                    }

                    if ($Curlopt == 'CURLOPT_HTTPHEADER') {
                        $httpheaders = explode('|', $Curloptvalue);
                        if ($this->Curly['conf']['debug'] == 1) {
                            echo $Curlopt . PHP_EOL;
                            print_r($Curloptvalue) . PHP_EOL;
                            print_r($httpheaders);
                            echo PHP_EOL;
                        }
                        curl_setopt($this->cURL, $this->Curly['curlopt_constants'][$Curlopt], $httpheaders);
                    } elseif ($Curlopt == 'CURLOPT_HTTPAUTH') {
                        if (isset($this->Curly['curlauth_constants'][$Curloptvalue])) {
                            curl_setopt($cURL, $this->Curly['curlopt_constants'][$Curlopt], $this->Curly['curlauth_constants'][$Curloptvalue]);
                        }
                    } elseif ($Curlopt == 'CURLOPT_FILE') {
                        /* Open for writing only; place the file pointer at the beginning of the file and truncate the file to zero length. If the file does not exist, attempt to create it. */
                        if (!$this->CurlOptFileHandle = fopen($Curloptvalue, 'w')) {
                            $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                            $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] = $Curlopt . ' - ' . $Curloptvalue . ' fopen failed';
                            return;
                        } else {
                            curl_setopt($this->cURL, $this->Curly['curlopt_constants'][$Curlopt], $this->CurlOptFileHandle);
                        }
                    } else {
                        curl_setopt($this->cURL, $this->Curly['curlopt_constants'][$Curlopt], $Curloptvalue);
                    }

                }

                $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_setopt'][$Curlopt] = $Curloptvalue;
            }

        }
        return;
    }

    private function get_parsepattern($testcasefile, $caseid, $Curlopt, $Curloptvalue, $requests)
    {
        $nbmatches = 0;
        $found = 0;

        if (preg_match_all('/{get_parsepattern:([a-zA-Z0-9\_\-]+.xml:\d+:\d+:\d+)}/', $Curloptvalue, $matches)) {

            $nbmatches = count($matches[1]);

            if ($nbmatches != $requests) {
                $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
                $message = 'get_parsepattern ' . $Curlopt . ' ' . $Curloptvalue . ' failed - syntax error ';
            } else {
                for ($i = 0; $i <= ($nbmatches - 1); $i++) {
                    $search = $matches[1][$i];
                    list($parsepattern_testcasefile, $parsepattern_caseid, $parsepatternid, $pattern_id) = explode(':', $search);
                    if (!empty($this->Curly['parsepattern_results'][$testcasefile]['case'][$parsepattern_caseid][$parsepatternid][$pattern_id])) {
                        $found++;
                        $replacement = $this->Curly['parsepattern_results'][$parsepattern_testcasefile]['case'][$parsepattern_caseid][$parsepatternid][$pattern_id];
                        $Curloptvalue = preg_replace("/{get_parsepattern:$search}/", $replacement, $Curloptvalue);
                    }
                }
                if ($found != $requests) {
                    $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
                    $message = 'get_parsepattern ' . $Curlopt . ' ' . $Curloptvalue . ' failed - requests ' . $requests . ' - found ' . $found . ' ';
                } else {
                    $status = $this->Curly['conf']['exit_codes']['OK'];
                    $message = 'get_parsepattern ok';
                }
            }
        } else {
            $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
            $message = 'get_parsepattern ' . $Curlopt . ' ' . $Curloptvalue . ' failed - syntax error ';
        }

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'get_parsepattern:' . $Curlopt . ' ' . $Curloptvalue . PHP_EOL;
            print_r($matches);
            echo 'requests: ' . $requests . PHP_EOL;
            echo 'nbmatches: ' . $nbmatches . PHP_EOL;
            echo 'found: ' . $found . PHP_EOL;
            echo $Curlopt . ' ' . $Curloptvalue . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo PHP_EOL;
        }
        return array($Curloptvalue, $status, $message);
    }

    private function ExecChecks($testcasefile, $casekey, $case, $caseid)
    {
        foreach ($case['checks'] as $checkname => $checkarg) {

            switch ($checkname) {

                case 'status_code':

                    foreach ($checkarg as $checkkey => $args) {

                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_statuscode($testcasefile, $casekey, $checkname, $checkkey, $args);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }
                    break;

                case 'primary_ip':

                    foreach ($checkarg as $checkkey => $args) {
                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_primary_ip($testcasefile, $casekey, $checkname, $checkkey, $args);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }
                    break;

                case 'redirect_count':

                    foreach ($checkarg as $checkkey => $args) {
                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_redirect_count($testcasefile, $casekey, $checkname, $checkkey, $args);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }
                    break;

                case 'response_time':

                    foreach ($checkarg as $checkkey => $args) {
                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_response_time($testcasefile, $casekey, $checkname, $checkkey, $args);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }
                    break;

                case 'matchpattern':

                    foreach ($checkarg as $checkkey => $args) {
                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_matchpattern($testcasefile, $casekey, $checkname, $checkkey, $args);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }
                    break;

                case 'nomatchpattern':

                    foreach ($checkarg as $checkkey => $args) {
                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_nomatchpattern($testcasefile, $casekey, $checkname, $checkkey, $args);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }
                    break;

                case 'parsepattern':

                    $parsepatternid = 0;
                    foreach ($checkarg as $checkkey => $args) {
                        $parsepatternid++;
                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_parsepattern($testcasefile, $casekey, $checkname, $checkkey, $args, $caseid, $parsepatternid);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }

                    break;

                case 'matchpatterncount':

                    foreach ($checkarg as $checkkey => $args) {
                        $this->Curly['result']['counters']['checksrun']++;
                        list($status, $checkmessage) = $this->check_matchpatterncount($testcasefile, $casekey, $checkname, $checkkey, $args);
                        switch ($status) {
                            case 3:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_unknown'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_unknown'] = 1;
                                $this->Curly['result']['counters']['checksunknown']++;
                                return;
                                break;
                            case 2:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_critical'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_critical'] = 1;
                                $this->Curly['result']['counters']['checkscritical']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 1:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_warning'] .= $checkmessage . ' ';
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['is_warning'] = 1;
                                $this->Curly['result']['counters']['checkswarning']++;
                                if ($this->Curly['conf']['break_on_error'] == 1) {
                                    return;
                                }
                                break;
                            case 0:
                                $this->Curly['result']['testcases'][$testcasefile][$casekey]['output_ok'] .= $checkmessage . ' ';
                                $this->Curly['result']['counters']['checksok']++;
                                break;
                        }
                    }

                    break;

            }
        }

        return;

    }

    private function check_statuscode($testcasefile, $casekey, $checkname, $checkkey, $args)
    {
        $code = $args['code'];
        if (!isset($this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['http_code'])) {
            $result = "unknown";
            $message = $checkname . '=' . $result;
            $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
        } else {
            $result = $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['http_code'];
            if ($result != $code) {
                $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
                if (isset($args['onfail_status'])) {
                    $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
                }
                $status = $fail_status;
                if (isset($args['errormessage'])) {
                    $message = $checkname . '=' . $args['errormessage'];
                } else {
                    $message = $checkname . '=' . $result;
                }
            } else {
                $status = $this->Curly['conf']['exit_codes']['OK'];
                $message = $checkname . '=' . $result;
            }
        }
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['code'] = $code;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = $result;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            echo 'code: ' . $code . PHP_EOL;
            echo 'result: ' . $result . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }

        return array($status, $message);
    }

    private function check_primary_ip($testcasefile, $casekey, $checkname, $checkkey, $args)
    {
        $ip = $args['ip'];
        if (!isset($this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['primary_ip'])) {
            $result = "unknown";
            $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
            $message = $checkname . '=' . $result;
        } else {
            $result = $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['primary_ip'];
            if ($result != $ip) {
                $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
                if (isset($args['onfail_status'])) {
                    $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
                }
                $status = $fail_status;
                if (isset($args['errormessage'])) {
                    $message = $checkname . '=' . $args['errormessage'];
                } else {
                    $message = $checkname . '=' . $result;
                }

            } else {
                $status = $this->Curly['conf']['exit_codes']['OK'];
                $message = $checkname . '=' . $result;
            }
        }

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['ip'] = $ip;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = $result;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            echo 'ip: ' . $ip . PHP_EOL;
            echo 'result: ' . $result . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }

        return array($status, $message);
    }

    private function check_redirect_count($testcasefile, $casekey, $checkname, $checkkey, $args)
    {
        $redirect_count = $args['count'];
        if (!isset($this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['redirect_count'])) {
            $result = "unknown";
            $message = $checkname . '=' . $result;
            $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
        } else {
            $result = $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['redirect_count'];
            if ($result != $redirect_count) {
                $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
                if (isset($args['onfail_status'])) {
                    $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
                }
                $status = $fail_status;
                if (isset($args['errormessage'])) {
                    $message = $checkname . '=' . $args['errormessage'];
                } else {
                    $message = $checkname . '=' . $result;
                }
            } else {
                $status = $this->Curly['conf']['exit_codes']['OK'];
                $message = $checkname . '=' . $result;
            }
        }

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['count'] = $redirect_count;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = $result;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            echo 'count: ' . $redirect_count . PHP_EOL;
            echo 'result: ' . $result . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }

        return array($status, $message);
    }

    private function check_response_time($testcasefile, $casekey, $checkname, $checkkey, $args)
    {
        $response_time = $args['time'];
        if (!isset($this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['total_time'])) {
            $result = "unknown";
            $message = $checkname . '=' . $result;
            $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
        } else {
            $result = $this->Curly['result']['testcases'][$testcasefile][$casekey]['curl_getinfo']['total_time'];
            $casetime = substr($result, 0, 5);
            if ($result > $response_time) {
                $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
                if (isset($args['onfail_status'])) {
                    $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
                }
                $status = $fail_status;
                if (isset($args['errormessage'])) {
                    $message = $checkname . '=' . $args['errormessage'];
                } else {
                    $message = $checkname . '=' . $result;
                }
            } else {
                $status = $this->Curly['conf']['exit_codes']['OK'];
                $message = $checkname . '=' . $result;
            }
            /*'label'=value[UOM];[warn];[crit];[min];[max]*/
            /* With response_time check we set casename warn;crit perfdatas */
            if ($status == 1) {
                $this->Curly['result']['output']['perfdatas'] .= $this->Curly['result']['testcases'][$testcasefile][$casekey]['casename'] . '=' . $casetime . ';' . $response_time . ';0;0;0 ';
            } else {
                $this->Curly['result']['output']['perfdatas'] .= $this->Curly['result']['testcases'][$testcasefile][$casekey]['casename'] . '=' . $casetime . ';0;' . $response_time . ';0;0 ';
            }
        }
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['time'] = $response_time;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = $result;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            echo 'time: ' . $response_time . PHP_EOL;
            echo 'result: ' . $result . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }

        return array($status, $message);
    }

    private function check_matchpattern($testcasefile, $casekey, $checkname, $checkkey, $args)
    {
        $pattern = $args['pattern'];
        $source = $args['source'];

        if (preg_match($pattern, $this->Curly['result']['testcases'][$testcasefile][$casekey][$source], $matches)) {
            $result = $matches[0];
            $count = count($matches[0]);
            $status = $this->Curly['conf']['exit_codes']['OK'];
            $message = $checkname . ($checkkey + 1) . '=' . $count;
        } else {
            $result = '';
            $count = 0;
            $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
            if (isset($args['onfail_status'])) {
                $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
            }
            $status = $fail_status;
            if (isset($args['errormessage'])) {
                $message = $checkname . ($checkkey + 1) . '=' . $args['errormessage'];
            } else {
                $message = $checkname . ($checkkey + 1) . '=' . $count;
            }
        }

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['pattern'] = $pattern;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['source'] = $source;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = $result;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            print_r($matches);
            echo 'pattern: ' . $pattern . PHP_EOL;
            echo 'source: ' . $source . PHP_EOL;
            echo 'result: ' . $result . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }
        return array($status, $message);
    }

    private function check_nomatchpattern($testcasefile, $casekey, $checkname, $checkkey, $args)
    {
        $pattern = $args['pattern'];
        $source = $args['source'];

        if (preg_match($pattern, $this->Curly['result']['testcases'][$testcasefile][$casekey][$source], $matches)) {
            $result = $matches[0];
            $count = count($matches[0]);
            $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
            if (isset($args['onfail_status'])) {
                $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
            }
            $status = $fail_status;
            if (isset($args['errormessage'])) {
                $message = $checkname . ($checkkey + 1) . '=' . $args['errormessage'];
            } else {
                $message = $checkname . ($checkkey + 1) . '=' . $count;
            }
        } else {
            $result = '';
            $count = 0;
            $message = $checkname . ($checkkey + 1) . '=' . $count;
            $status = $this->Curly['conf']['exit_codes']['OK'];
        }

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['pattern'] = $pattern;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['source'] = $source;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = $result;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            print_r($matches);
            echo 'pattern: ' . $pattern . PHP_EOL;
            echo 'source: ' . $source . PHP_EOL;
            echo 'result: ' . $result . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }
        return array($status, $message);
    }

    private function check_parsepattern($testcasefile, $casekey, $checkname, $checkkey, $args, $caseid, $parsepatternid)
    {
        $pattern = $args['pattern'];
        $source = $args['source'];

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = '';
        $nbmatches = 0;
        $patternmatches = 0;
        $errors = 0;

        if (preg_match($pattern, $this->Curly['result']['testcases'][$testcasefile][$casekey][$source], $matches)) {
            $nbmatches = count($matches);
            if ($nbmatches >= 2) {
                for ($patternid = 1; $patternid <= ($nbmatches - 1); $patternid++) {
                    $parseresult = $matches[$patternid];
                    if ($parseresult != '') {
                        $patternmatches++;
                        $this->Curly['parsepattern_results'][$testcasefile]['case'][$caseid][$parsepatternid][$patternid] = $parseresult;
                        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] .= $parseresult . ' ';
                    } else {
                        $errors++;
                    }
                }
            } else {
                $errors++;
            }
            if ($errors == 0) {
                $status = $this->Curly['conf']['exit_codes']['OK'];
                $message = $checkname . ($checkkey + 1) . '=' . ($nbmatches - 1);
            } else {
                $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
                $message = $checkname . ($checkkey + 1) . '=' . $patternmatches;
            }
        } else {
            $status = $this->Curly['conf']['exit_codes']['UNKNOWN'];
            $message = $checkname . ($checkkey + 1) . '=' . $nbmatches;
        }

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['pattern'] = $pattern;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['source'] = $source;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            print_r($matches);
            echo 'pattern: ' . $pattern . PHP_EOL;
            echo 'source: ' . $source . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }

        return array($status, $message);

    }

    private function check_matchpatterncount($testcasefile, $casekey, $checkname, $checkkey, $args)
    {
        $pattern = $args['pattern'];
        $source = $args['source'];
        $matchcount = $args['matchcount'];

        if (preg_match_all($pattern, $this->Curly['result']['testcases'][$testcasefile][$casekey][$source], $matches)) {
            $count = count($matches[0]);
            if ($count == $matchcount) {
                $status = $this->Curly['conf']['exit_codes']['OK'];
                $message = $checkname . ($checkkey + 1) . '=' . $count;
            } else {
                $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
                if (isset($args['onfail_status'])) {
                    $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
                }
                $status = $fail_status;
                if (isset($args['errormessage'])) {
                    $message = $checkname . ($checkkey + 1) . '=' . $args['errormessage'];
                } else {
                    $message = $checkname . ($checkkey + 1) . '=' . $count;
                }

            }

        } else {
            $count = 0;
            $fail_status = $this->Curly['conf']['exit_codes']['CRITICAL'];
            if (isset($args['onfail_status'])) {
                $fail_status = $this->Curly['conf']['exit_codes'][$args['onfail_status']];
            }
            $status = $fail_status;
            if (isset($args['errormessage'])) {
                $message = $checkname . ($checkkey + 1) . '=' . $args['errormessage'];
            } else {
                $message = $checkname . ($checkkey + 1) . '=' . $count;
            }
        }

        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['pattern'] = $pattern;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['source'] = $source;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['matchcount'] = $matchcount;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['result'] = $count;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['message'] = $message;
        $this->Curly['result']['testcases'][$testcasefile][$casekey]['checks'][$checkname][$checkkey]['status'] = $status;

        if ($this->Curly['conf']['debug'] == 1) {
            echo PHP_EOL . PHP_EOL;
            echo 'Check:' . $checkname . PHP_EOL;
            print_r($matches);
            echo 'pattern: ' . $pattern . PHP_EOL;
            echo 'source: ' . $source . PHP_EOL;
            echo 'matchcount: ' . $matchcount . PHP_EOL;
            echo 'result: ' . $count . PHP_EOL;
            echo 'message: ' . $message . PHP_EOL;
            echo 'status: ' . $status . PHP_EOL;
            echo PHP_EOL;
        }

        return array($status, $message);
    }

    private function endtasks()
    {
        $this->deletecookies();
        $this->CurlyTime();
        /* Each case time is added here in RunCase or check_response_time */
        $this->Curly['result']['output']['perfdatas'] .= 'Curlytime=' . $this->Curly['result']['counters']['CurlyTime'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' casestime=' . $this->Curly['result']['counters']['casestime'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' casesrun=' . $this->Curly['result']['counters']['casesrun'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' casesok=' . $this->Curly['result']['counters']['casesok'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' caseswarning=' . $this->Curly['result']['counters']['caseswarning'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' casescritical=' . $this->Curly['result']['counters']['casescritical'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' casesunknown=' . $this->Curly['result']['counters']['casesunknown'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' checksrun=' . $this->Curly['result']['counters']['checksrun'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' checksok=' . $this->Curly['result']['counters']['checksok'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' checkswarning=' . $this->Curly['result']['counters']['checkswarning'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' checkscritical=' . $this->Curly['result']['counters']['checkscritical'] . ';0;0;0;0';
        $this->Curly['result']['output']['perfdatas'] .= ' checksunknown=' . $this->Curly['result']['counters']['checksunknown'] . ';0;0;0;0';

        if ($this->Curly['result']['counters']['casesunknown'] != 0) {
            $this->Curly['result']['exit_status'] = 3;
        } elseif ($this->Curly['result']['counters']['casescritical'] != 0) {
            $this->Curly['result']['exit_status'] = 2;
        } elseif ($this->Curly['result']['counters']['caseswarning'] != 0) {
            $this->Curly['result']['exit_status'] = 1;
        } else {
            $this->Curly['result']['exit_status'] = 0;
        }

        switch ($this->Curly['result']['exit_status']) {
            case 3:
                $output = 'Curly UNKNOWN - ' . $this->Curly['result']['output']['UNKNOWN'] . ' |' . $this->Curly['result']['output']['perfdatas'];
                break;
            case 2:
                $output = 'Curly CRITICAL - ' . $this->Curly['result']['output']['CRITICAL'] . ' |' . $this->Curly['result']['output']['perfdatas'];
                break;
            case 1:
                $output = 'Curly WARNING - ' . $this->Curly['result']['output']['WARNING'] . ' |' . $this->Curly['result']['output']['perfdatas'];
                break;
            case 0:
                if ($this->Curly['conf']['output_ok_length'] == 'long') {
                    $output = 'Curly OK - ' . $this->Curly['result']['output']['OK'] . ' |' . $this->Curly['result']['output']['perfdatas'];
                } else {
                    $output = 'Curly OK |' . $this->Curly['result']['output']['perfdatas'];
                }
                break;
        }

        $output = preg_replace('/,?\s+/', ' ', $output);
        $this->Curly['result']['output']['stdout'] = $output;
        $this->LogResults();
    }

    private function deletecookies()
    {
        $cookies = array('CURLOPT_COOKIEFILE', 'CURLOPT_COOKIEJAR');
        if ($this->Curly['conf']['deletecookies'] == 1) {
            foreach ($cookies as $key) {
                if (isset($this->Curly['conf']['curl_setopt'][$key])) {
                    if (is_file($this->Curly['conf']['curl_setopt'][$key])) {
                        @unlink($this->Curly['conf']['curl_setopt'][$key]);
                    }
                }
            }
        }
    }

    private function CurlyTime()
    {
        $this->Curly['result']['counters']['CurlyEndTime'] = microtime(true);
        $CurlyTime = substr(($this->Curly['result']['counters']['CurlyEndTime'] - $this->Curly['result']['counters']['CurlyStartTime']), 0, 5);
        $this->Curly['result']['counters']['CurlyTime'] = $CurlyTime;
    }

    private function LogResults()
    {
        if ($this->Curly['conf']['logenable'] == 1) {
            $this->LogHandle = false;
            /* Open for writing only; place the file pointer at the end of the file. If the file does not exist, attempt to create it.  */
            if (!$this->LogHandle = fopen($this->Curly['conf']['logsdir'] . '/' . $this->Curly['conf']['logfile'], 'a')) {
                echo 'Curly UNKNOWN - ' . $this->Curly['conf']['logsdir'] . '/' . $this->Curly['conf']['logfile'] . ' fopen failed' . PHP_EOL;
                exit($this->Curly['conf']['exit_codes']['UNKNOWN']);
            }

            $logdate = date("Y-m-d H:i:s");

            fwrite($this->LogHandle, "\n\n\n\n################################## $logdate ##################################\n");

            foreach ($this->Curly['result']['testcases'] as $k1 => $v1) {
                fwrite($this->LogHandle, "################\n");
                fwrite($this->LogHandle, "# testcasefile #\n");
                fwrite($this->LogHandle, "################\n");
                fwrite($this->LogHandle, $k1 . "\n");
                foreach ($v1 as $k2 => $v2) {
                    fwrite($this->LogHandle, "##########\n");
                    fwrite($this->LogHandle, "# caseid #\n");
                    fwrite($this->LogHandle, "##########\n");
                    fwrite($this->LogHandle, $v2['caseid'] . "\n");
                    fwrite($this->LogHandle, "############\n");
                    fwrite($this->LogHandle, "# casename #\n");
                    fwrite($this->LogHandle, "############\n");
                    fwrite($this->LogHandle, $v2['casename'] . "\n");
                    fwrite($this->LogHandle, "####################\n");
                    fwrite($this->LogHandle, "# cURL curl_setopt #\n");
                    fwrite($this->LogHandle, "####################\n");
                    foreach ($v2['curl_setopt'] as $a => $b) {
                        fwrite($this->LogHandle, $a . " " . $b . "\n");
                    }
                    fwrite($this->LogHandle, "################\n");
                    fwrite($this->LogHandle, "# cURL getinfo #\n");
                    fwrite($this->LogHandle, "################\n");
                    foreach ($v2['curl_getinfo'] as $type => $value) {
                        if ($type == "certinfo") {
                            foreach ($value as $c3 => $c4) {
                                foreach ($c4 as $d1 => $d2) {
                                    if (!is_array($d2)) {
                                        fwrite($this->LogHandle, $d1 . " " . $d2 . "\n");
                                    } else {
                                        foreach ($d2 as $e1 => $e2) {
                                            fwrite($this->LogHandle, $e1 . " " . $e2 . "\n");
                                        }
                                    }
                                }
                            }

                        } else {
                            fwrite($this->LogHandle, $type . " " . $value . "\n");
                        }
                    }
                    fwrite($this->LogHandle, "################\n");
                    fwrite($this->LogHandle, "# cURL Verbose #\n");
                    fwrite($this->LogHandle, "################\n");
                    fwrite($this->LogHandle, $v2['curlverbose'] . "\n");
                    fwrite($this->LogHandle, "########\n");
                    fwrite($this->LogHandle, "# cURL #\n");
                    fwrite($this->LogHandle, "########\n");
                    fwrite($this->LogHandle, $v2['curl'] . "\n");
                    fwrite($this->LogHandle, "##########\n");
                    fwrite($this->LogHandle, "# Checks #\n");
                    fwrite($this->LogHandle, "##########\n");
                    foreach ($v2['checks'] as $a => $b) {
                        fwrite($this->LogHandle, $a . "\n");
                        foreach ($b as $b1 => $b2) {
                            foreach ($b2 as $c1 => $c2) {
                                fwrite($this->LogHandle, $c1 . " " . $c2 . "\n");
                            }
                        }
                    }
                    fwrite($this->LogHandle, "\n\n");
                }
            }

            fwrite($this->LogHandle, "##########\n");
            fwrite($this->LogHandle, "# output #\n");
            fwrite($this->LogHandle, "##########\n");
            fwrite($this->LogHandle, $this->Curly['result']['output']['stdout']);
            fclose($this->LogHandle);
        }
    }
}

;

$Curly = new Curly;
$Curly->CurlyExec($argv);
